@@ -21,13 +21,15 @@ import logging
 import os
 import socket
 
-from oslo.config import cfg
-from oslo.utils import excutils
-from oslo.utils import units
 from oslo.vmware import api
+from oslo.vmware import constants
+from oslo_config import cfg
+from oslo_utils import excutils
+from oslo_utils import units
 import six.moves.urllib.parse as urlparse
 
 import glance_store
+from glance_store import capabilities
 from glance_store import exceptions
 from glance_store.i18n import _
 from glance_store.i18n import _LE
@@ -38,7 +40,6 @@ LOG = logging.getLogger(__name__)
 
 MAX_REDIRECTS = 5
 DEFAULT_STORE_IMAGE_DIR = '/openstack_glance'
-DEFAULT_ESX_DATACENTER_PATH = 'ha-datacenter'
 DS_URL_PREFIX = '/folder'
 STORE_SCHEME = 'vsphere'
 
@@ -57,7 +58,7 @@ _VMWARE_OPTS = [
                       'VMware ESX/VC server.'),
                secret=True),
     cfg.StrOpt('vmware_datacenter_path',
-               default=DEFAULT_ESX_DATACENTER_PATH,
+               default=constants.ESX_DATACENTER_PATH,
                help=_('Inventory path to a datacenter. '
                       'If the vmware_server_host specified is an ESX/ESXi, '
                       'the vmware_datacenter_path is optional. If specified, '
@@ -119,15 +120,6 @@ class _Reader(object):
         self.checksum.update(result)
         return result
 
-    def rewind(self):
-        try:
-            self.data.seek(0)
-            self._size = 0
-            self.checksum = hashlib.md5()
-        except IOError:
-            with excutils.save_and_reraise_exception():
-                LOG.exception(_LE('Failed to rewind image content'))
-
     @property
     def size(self):
         return self._size
@@ -229,8 +221,20 @@ class StoreLocation(location.StoreLocation):
 class Store(glance_store.Store):
     """An implementation of the VMware datastore adapter."""
 
+    _CAPABILITIES = capabilities.RW_ACCESS
     OPTIONS = _VMWARE_OPTS
     WRITE_CHUNKSIZE = units.Mi
+    # FIXME(arnaud): re-visit this code once the store API is cleaned up.
+    _VMW_SESSION = None
+
+    def reset_session(self, force=False):
+        if Store._VMW_SESSION is None or force:
+            Store._VMW_SESSION = api.VMwareAPISession(
+                self.server_host, self.server_username, self.server_password,
+                self.api_retry_count, self.tpoll_interval)
+        return Store._VMW_SESSION
+
+    session = property(reset_session)
 
     def get_schemes(self):
         return (STORE_SCHEME,)
@@ -247,12 +251,6 @@ class Store(glance_store.Store):
             raise exceptions.BadStoreConfiguration(
                 store_name='vmware_datastore', reason=msg)
 
-    def _create_session(self):
-        self._session = api.VMwareAPISession(
-            self.server_host, self.server_username, self.server_password,
-            self.api_retry_count, self.tpoll_interval)
-        self._service_content = self._session.vim.service_content
-
     def configure(self):
         self._sanity_check()
         self.scheme = STORE_SCHEME
@@ -262,7 +260,6 @@ class Store(glance_store.Store):
         self.api_retry_count = self.conf.glance_store.vmware_api_retry_count
         self.tpoll_interval = self.conf.glance_store.vmware_task_poll_interval
         self.api_insecure = self.conf.glance_store.vmware_api_insecure
-        self._create_session()
         super(Store, self).configure()
 
     def configure_add(self):
@@ -270,14 +267,14 @@ class Store(glance_store.Store):
         self.datastore_name = self._option_get('vmware_datastore_name')
         global _datastore_info_valid
         if not _datastore_info_valid:
-            search_index_moref = self._service_content.searchIndex
+            search_index_moref = (
+                self.session.vim.service_content.searchIndex)
 
             inventory_path = ('%s/datastore/%s'
                               % (self.datacenter_path, self.datastore_name))
-            ds_moref = self._session.invoke_api(self._session.vim,
-                                                'FindByInventoryPath',
-                                                search_index_moref,
-                                                inventoryPath=inventory_path)
+            ds_moref = self.session.invoke_api(
+                self.session.vim, 'FindByInventoryPath',
+                search_index_moref, inventoryPath=inventory_path)
             if ds_moref is None:
                 msg = (_("Could not find datastore %(ds_name)s "
                          "in datacenter %(dc_path)s")
@@ -299,12 +296,16 @@ class Store(glance_store.Store):
                 store_name='vmware_datastore', reason=reason)
         return result
 
-    def _build_vim_cookie_header(self, vim_cookies):
+    def _build_vim_cookie_header(self, verify_session=False):
         """Build ESX host session cookie header."""
+        if verify_session and not self.session.is_current_session_active():
+            self.reset_session(force=True)
+        vim_cookies = self.session.vim.client.options.transport.cookiejar
         if len(list(vim_cookies)) > 0:
             cookie = list(vim_cookies)[0]
             return cookie.name + '=' + cookie.value
 
+    @capabilities.check
     def add(self, image_id, image_file, image_size, context=None):
         """Stores an image file with supplied identifier to the backend
         storage system and returns a tuple containing information
@@ -337,40 +338,50 @@ class Store(glance_store.Store):
                              'datastore_name': self.datastore_name,
                              'image_id': image_id}, self.conf)
         # NOTE(arnaud): use a decorator when the config is not tied to self
-        for i in range(self.api_retry_count + 1):
-            cookie = self._build_vim_cookie_header(
-                self._session.vim.client.options.transport.cookiejar)
-            headers = dict(headers.items() + {'Cookie': cookie}.items())
-            try:
-                conn = self._get_http_conn('PUT', loc, headers,
-                                           content=image_file)
-                res = conn.getresponse()
-            except Exception:
-                with excutils.save_and_reraise_exception():
-                    LOG.exception(_LE('Failed to upload content of image '
-                                      '%(image)s'), {'image': image_id})
+        cookie = self._build_vim_cookie_header(True)
+        headers = dict(headers.items() + {'Cookie': cookie}.items())
+        conn_class = self._get_http_conn_class()
+        conn = conn_class(loc.server_host)
+        url = urlparse.quote('%s?%s' % (loc.path, loc.query))
+        try:
+            conn.request('PUT', url, image_file, headers)
+        except IOError as e:
+            # When a session is not authenticated, the socket is closed by
+            # the server after sending the response. httplib has an open
+            # issue with https that raises Broken Pipe
+            # error instead of returning the response.
+            # See http://bugs.python.org/issue16062. Here, we log the error
+            # and continue to look into the response.
+            msg = _LE('Communication error sending http %(method)s request'
+                      'to the url %(url)s.\n'
+                      'Got IOError %(e)s') % {'method': 'PUT',
+                                              'url': url,
+                                              'e': e}
+            LOG.error(msg)
+        except Exception:
+            with excutils.save_and_reraise_exception():
+                LOG.exception(_LE('Failed to upload content of image '
+                                  '%(image)s'), {'image': image_id})
+        res = conn.getresponse()
+        if res.status == httplib.CONFLICT:
+            raise exceptions.Duplicate(_("Image file %(image_id)s already "
+                                         "exists!") %
+                                       {'image_id': image_id})
 
-            if res.status == httplib.UNAUTHORIZED:
-                self._create_session()
-                image_file.rewind()
-                continue
-
-            if res.status == httplib.CONFLICT:
-                raise exceptions.Duplicate(_("Image file %(image_id)s already "
-                                             "exists!") %
-                                           {'image_id': image_id})
-
-            if res.status not in (httplib.CREATED, httplib.OK):
-                msg = (_LE('Failed to upload content of image %(image)s') %
-                       {'image': image_id})
-                LOG.error(msg)
-                raise exceptions.UnexpectedStatus(status=res.status,
-                                                  body=res.read())
-            break
+        if res.status not in (httplib.CREATED, httplib.OK):
+            msg = (_LE('Failed to upload content of image %(image)s. '
+                       'The request returned an unexpected status: %(status)s.'
+                       '\nThe response body:\n%(body)s') %
+                   {'image': image_id,
+                    'status': res.status,
+                    'body': res.body})
+            LOG.error(msg)
+            raise exceptions.BackendException(msg)
 
         return (loc.get_uri(), image_file.size,
                 image_file.checksum.hexdigest(), {})
 
+    @capabilities.check
     def get(self, location, offset=0, chunk_size=None, context=None):
         """Takes a `glance_store.location.Location` object that indicates
         where to find the image file, and returns a tuple of generator
@@ -401,6 +412,7 @@ class Store(glance_store.Store):
         """
         return self._query(location, 'HEAD')[2]
 
+    @capabilities.check
     def delete(self, location, context=None):
         """Takes a `glance_store.location.Location` object that indicates
         where to find the image file to delete
@@ -412,19 +424,18 @@ class Store(glance_store.Store):
         file_path = '[%s] %s' % (
             self.datastore_name,
             location.store_location.path[len(DS_URL_PREFIX):])
-        search_index_moref = self._service_content.searchIndex
-        dc_moref = self._session.invoke_api(self._session.vim,
-                                            'FindByInventoryPath',
-                                            search_index_moref,
-                                            inventoryPath=self.datacenter_path)
-        delete_task = self._session.invoke_api(
-            self._session.vim,
+        search_index_moref = self.session.vim.service_content.searchIndex
+        dc_moref = self.session.invoke_api(
+            self.session.vim, 'FindByInventoryPath', search_index_moref,
+            inventoryPath=self.datacenter_path)
+        delete_task = self.session.invoke_api(
+            self.session.vim,
             'DeleteDatastoreFile_Task',
-            self._service_content.fileManager,
+            self.session.vim.service_content.fileManager,
             name=file_path,
             datacenter=dc_moref)
         try:
-            self._session.wait_for_task(delete_task)
+            self.session.wait_for_task(delete_task)
         except Exception:
             with excutils.save_and_reraise_exception():
                 LOG.exception(_LE('Failed to delete image %(image)s '
@@ -439,8 +450,7 @@ class Store(glance_store.Store):
         loc = location.store_location
         # NOTE(arnaud): use a decorator when the config is not tied to self
         for i in range(self.api_retry_count + 1):
-            cookie = self._build_vim_cookie_header(
-                self._session.vim.client.options.transport.cookiejar)
+            cookie = self._build_vim_cookie_header()
             headers = {'Cookie': cookie}
             try:
                 conn = self._get_http_conn(method, loc, headers)
@@ -452,7 +462,7 @@ class Store(glance_store.Store):
                                                      location.image_id})
             if resp.status >= 400:
                 if resp.status == httplib.UNAUTHORIZED:
-                    self._create_session()
+                    self.reset_session(force=True)
                     continue
                 if resp.status == httplib.NOT_FOUND:
                     reason = _('VMware datastore could not find image at URI.')
